@@ -120,7 +120,7 @@ class AzureDatalakeFileSystem(AbstractFileSystem):
 
     @staticmethod
     def _get_kwargs_from_urls(paths):
-        """ Get the store_name from the urlpath and pass to storage_options """
+        """Get the store_name from the urlpath and pass to storage_options"""
         ops = infer_storage_options(paths)
         out = {}
         if ops.get("host", None):
@@ -165,7 +165,7 @@ class AzureDatalakeFileSystem(AbstractFileSystem):
         return info
 
     def _trim_filename(self, fn, **kwargs):
-        """ Determine what kind of filestore this is and return the path """
+        """Determine what kind of filestore this is and return the path"""
         so = infer_storage_options(fn)
         fileparts = so["path"]
         return fileparts
@@ -287,6 +287,25 @@ class AzureDatalakeFile(AzureDLFile):
         return self.loc
 
 
+# https://github.com/Azure/azure-sdk-for-python/issues/11419#issuecomment-628143480
+def make_callback(key, callback):
+    if callback is None:
+        return None
+
+    sent_total = False
+
+    def wrapper(response):
+        nonlocal sent_total
+
+        current = response.context.get(key)
+        total = response.context["data_stream_total"]
+        if not sent_total:
+            callback.set_size(total)
+        callback.absolute_update(current)
+
+    return wrapper
+
+
 class AzureBlobFileSystem(AsyncFileSystem):
     """
     Access Azure Datalake Gen2 and Azure Storage if it were a file system using Multiprotocol Access
@@ -529,7 +548,9 @@ class AzureBlobFileSystem(AsyncFileSystem):
                     conn_str=self.connection_string
                 )
             elif self.account_name:
-                self.account_url: str = f"https://{self.account_name}.blob.core.windows.net"
+                self.account_url: str = (
+                    f"https://{self.account_name}.blob.core.windows.net"
+                )
                 creds = [self.credential, self.account_key]
                 if any(creds):
                     self.service_client = [
@@ -923,6 +944,12 @@ class AzureBlobFileSystem(AsyncFileSystem):
                         and data["metadata"]["is_directory"] == "false"
                     ):
                         data.update({"type": "file"})
+                    elif (
+                        "hdi_isfolder" in data["metadata"].keys()
+                        and data["metadata"]["hdi_isfolder"] == "true"
+                    ):
+                        data.update({"type": "directory"})
+                        data.update({"size": None})
                     else:
                         pass
             if return_glob:
@@ -1525,7 +1552,9 @@ class AzureBlobFileSystem(AsyncFileSystem):
             raise FileNotFoundError
         return list(sorted(out))
 
-    async def _put_file(self, lpath, rpath, delimiter="/", overwrite=False, **kwargws):
+    async def _put_file(
+        self, lpath, rpath, delimiter="/", overwrite=False, callback=None, **kwargws
+    ):
         """
         Copy single file to remote
 
@@ -1546,7 +1575,12 @@ class AzureBlobFileSystem(AsyncFileSystem):
                         container_name, path
                     ) as bc:
                         await bc.upload_blob(
-                            f1, overwrite=overwrite, metadata={"is_directory": "false"}
+                            f1,
+                            overwrite=overwrite,
+                            metadata={"is_directory": "false"},
+                            raw_response_hook=make_callback(
+                                "upload_stream_current", callback
+                            ),
                         )
                 self.invalidate_cache()
             except ResourceExistsError:
@@ -1560,7 +1594,7 @@ class AzureBlobFileSystem(AsyncFileSystem):
     put_file = sync_wrapper(_put_file)
 
     async def _cp_file(self, path1, path2, **kwargs):
-        """ Copy the file at path1 to path2 """
+        """Copy the file at path1 to path2"""
         container1, path1 = self.split_path(path1, delimiter="/")
         container2, path2 = self.split_path(path2, delimiter="/")
 
@@ -1585,24 +1619,26 @@ class AzureBlobFileSystem(AsyncFileSystem):
         """Alias of :ref:`FilesystemSpec.get`."""
         return self.get(rpath, lpath, recursive=recursive, **kwargs)
 
-    async def _get_file(self, rpath, lpath, recursive=False, delimiter="/", **kwargs):
-        """ Copy single file remote to local """
-        files = await self._ls(rpath)
-        files = [f["name"] for f in files]
+    async def _get_file(
+        self, rpath, lpath, recursive=False, delimiter="/", callback=None, **kwargs
+    ):
+        """Copy single file remote to local"""
+        if os.path.isdir(lpath):
+            return
         container_name, path = self.split_path(rpath, delimiter=delimiter)
         try:
-            if await self._isdir(rpath):
-                os.makedirs(lpath, exist_ok=True)
-            else:
-                async with self.service_client.get_blob_client(
-                    container_name, path.rstrip(delimiter)
-                ) as bc:
-                    with open(lpath, "wb") as my_blob:
-                        stream = await bc.download_blob()
-                        data = await stream.readall()
-                        my_blob.write(data)
-        except Exception as e:
-            raise FileNotFoundError(f"File not found for {e}")
+            async with self.service_client.get_blob_client(
+                container_name, path.rstrip(delimiter)
+            ) as bc:
+                with open(lpath, "wb") as my_blob:
+                    stream = await bc.download_blob(
+                        raw_response_hook=make_callback(
+                            "download_stream_current", callback
+                        )
+                    )
+                    await stream.readinto(my_blob)
+        except ResourceNotFoundError as exception:
+            raise FileNotFoundError from exception
 
     get_file = sync_wrapper(_get_file)
 
@@ -1676,7 +1712,7 @@ class AzureBlobFileSystem(AsyncFileSystem):
 
 
 class AzureBlobFile(AbstractBufferedFile):
-    """ File-like operations on Azure Blobs """
+    """File-like operations on Azure Blobs"""
 
     DEFAULT_BLOCK_SIZE = 5 * 2 ** 20
 
